
@media all {
    .nav-pills .nav-link {
        transition: opacity 0.3s ease; /* Übergangseffekt */
    }

    .nav-pills .nav-link:not(.active) {
        opacity: 0.6; /* Starttransparenz für nicht aktive Links */
    }

    .nav-pills .nav-link:not(.active):hover {
        opacity: 1.0; /* Transparenz auf 100% beim Hover */
    }

    .no-opacity {
        position: relative;
        z-index: 1;
        opacity: 1 !important;
    }

    *,
    *::before,
    *::after {
        box-sizing: border-box;
    }

    .card-characters-administration,
    .card-corporation-administration,
    .card-alliance-administration {
        display: flex;
        min-width: 300px;
    }

    .card-characters {
        display: flex;
        flex-wrap: wrap;
        list-style: none;
        margin: 0 -1rem 2rem;
        padding: 0;
        justify-content: center;
    }

    .card-characters .card {
        position: relative;
        display: flex;
        flex-direction: column;
        min-width: 0;
        overflow-wrap: break-word;
        background-clip: border-box;
        background-color: rgb(100 100 100);
        border: 1px solid rgb(0 0 0 / 12.5%);
        border-radius: 0.25rem;
    }

    .card-characters .card-header,
    .card-characters .card-footer {
        padding: 0.75rem 1.25rem;
        margin-bottom: 0;
        background-color: rgb(0 0 0 / 3%);
        border-bottom: 1px solid rgb(0 0 0 / 12.5%);
        text-align: center;
    }

    .card-characters .card-footer {
        border-bottom: none;
        border-top: 1px solid rgb(0 0 0 / 12.5%);
    }

    .card-characters .card-header:first-child {
        border-radius: calc(0.25rem - 1px) calc(0.25rem - 1px) 0 0;
    }

    .card-characters .card-body {
        flex: 1 1 auto;
        min-height: 1px;
        padding: 1.25rem;
    }

    .card-characters .card-item {
        display: flex;
        padding: 1rem;
    }

    .card-characters .card-item .card {
        display: flex;
        flex-direction: column;
    }

    .card-characters img {
        max-width: 100%;
    }

    .card-characters .text-center {
        text-align: center;
    }

    @media (width >= 738px) {
        .card-characters .card-item {
            width: calc(100% / 2);
        }
    }

    @media (width >= 976px) {
        .card-characters .card-item {
            width: calc(100% / 3);
        }
    }

    @media (width >= 1390px) {
        .card-characters .card-item {
            width: calc(100% / 4);
        }
    }

    @media (width >= 1700px) {
        .card-characters .card-item {
            width: calc(100% / 5);
        }
    }

    @media (width >= 2000px) {
        .card-characters .card-item {
            width: calc(100% / 6);
        }
    }

    @media (width >= 2200px) {
        .card-characters .card-item {
            width: calc(100% / 7);
        }
    }

    @media (width >= 2400px) {
        .card-characters .card-item {
            width: calc(100% / 8);
        }
    }
}
