# Generated by Django 4.2.23 on 2025-07-22 04:34

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "ledger",
            "0014_alter_corporationaudit_options_alter_general_options_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="characteraudit",
            old_name="character",
            new_name="eve_character",
        ),
    ]
