# Generated by Django 4.2.11 on 2025-01-13 11:35

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ledger", "0006_characteraudit_character_name_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="characterplanetdetails",
            name="facilitys",
            field=models.JSONField(blank=True, default=None, null=True),
        ),
    ]
