{% extends 'ledger/base.html' %}
{% load i18n %}

{% block page_template_title %}
    Error Handler
{% endblock page_template_title %}

{% block page_topic %}
    <h1 class="page-header text-center">{% translate "Ledger - Error" %}</h1>
{% endblock page_topic %}

{% block ledger_block %}
    <div class="panel panel-primary">
        <div class="panel-heading">
             <div class="panel-title">{% translate "Index" %}</div>
        </div>

        <div class="panel-body">
            {{ error }}
        </div>
    </div>
{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_css %}
{% endblock %}

{% block extra_script %}
{% endblock %}
