# Generated by Django 4.2.23 on 2025-07-21 18:21

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ledger", "0013_delete_events"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="corporationaudit",
            options={
                "default_permissions": (),
                "permissions": (
                    ("corp_audit_manager", "Has Access to own Corporations."),
                    ("corp_audit_admin_manager", "Has access to all Corporations."),
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app, Ledger."),
                    ("advanced_access", "Can access Corporation and Alliance Ledger."),
                    ("manage_access", "Can manage Ledger."),
                ),
            },
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("wallet_division_names", "Divisions Names"),
                    ("wallet_division", "Divisions"),
                    ("wallet_journal", "Wallet Journal"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
    ]
