{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:index' %}">
        {% translate 'Character Ledger' %}
        {% if issues %} <span class="badge bg-warning">{{ issues}}</span>{% endif %}
    </a>
</li>

<li class="nav-item">
    <a class="nav-link" href="{% url 'ledger:planetary_ledger_index' %}">{% translate 'Planetary Ledger' %}</a>
</li>

{% if perms.ledger.advanced_access %}
    <li class="nav-item mt-2">
        |
    </li>

    <li class="nav-item">
        <a class="nav-link" href="{% url 'ledger:corporation_ledger_index' %}">{% translate 'Corporation Ledger' %}</a>
    </li>

    <li class="nav-item">
        <a class="nav-link" href="{% url 'ledger:alliance_ledger_index' %}">{% translate 'Alliance Ledger' %}</a>
    </li>
{% endif %}
