# Generated by Django 4.2.25 on 2025-10-17 05:40

# Django
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ledger", "0016_characterminingledger_price_per_unit"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eveentity",
            name="category",
            field=models.CharField(
                choices=[
                    ("alliance", "Alliance"),
                    ("corporation", "Corporation"),
                    ("character", "Character"),
                ],
                max_length=32,
                verbose_name="Category",
            ),
        ),
        migrations.AlterField(
            model_name="eveentity",
            name="eve_id",
            field=models.IntegerField(
                primary_key=True,
                serialize=False,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="ID",
            ),
        ),
        migrations.AlterField(
            model_name="eveentity",
            name="name",
            field=models.CharField(max_length=254, verbose_name="Name"),
        ),
    ]
