@media all {
    @keyframes l-shake {
        0%, 100% { transform: translateX(0); }
        25% { transform: translateX(-5px); }
        50% { transform: translateX(5px); }
        75% { transform: translateX(-5px); }
    }

    .l-shake {
        animation: l-shake 0.5s;
        animation-iteration-count: infinite;
    }

    .custom-line {
        height: 3px;
        border: none;
        background: linear-gradient(to right, rgb(0 170 255 / 100%) 0%, rgb(0 170 255 / 0%) 80%, rgb(0 170 255 / 0%) 100%);
        z-index: 1;
    }

    /* Style the Image Used to Trigger the Modal */
    .zoom {
        opacity: 0.7;
        border-radius: 10px;
        cursor: pointer;
        transition: 0.3s;
    }

    .zoom:hover {
        opacity: 1;
        transform: scale(1.1);
    }

    /* Custom CSS for tab mouseover */
    .nav-tabs .nav-link:hover {
        opacity: 0.8;
        color: white;
        border-color: white;
    }

    .aa-ledger-modal-header > h5 {
        border-bottom: 1px solid var(--bs-secondary-bg-subtle);
    }

    .dropdown-scrollable {
        max-height: 200px; /* Adjust the height as needed */
        overflow-y: auto;
    }

    .dropdown-item.active {
        background-color: rgb(248 249 250); /* Lighter background color */
        color: rgb(33 37 41); /* Dark text color */
    }

    .card-ledger {
        min-width: 1000px;
    }

    .rattingchart {
        height: 500px;
        margin-left: 50px;
        margin-top: 70px;
        margin-bottom: 50px;
    }

    .rattingbar {
        height: 500px;
    }
}


@media (width <= 1238px) {
    .rattingchart {
        height: 250px;
        margin-left: 10px;
        margin-bottom: 70px;
    }

    .rattingbar {
        height: 250px;
    }
}

@media (width >= 1238px) {
    .rattingchart {
        height: 280px;
        margin-left: 10px;
        margin-bottom: 70px;
    }

    .rattingbar {
        height: 280px;
    }
}

@media (width >= 1680px) {
    .rattingchart {
        height: 500px;
    }

    .rattingbar {
        height: 500px;
    }
}
